<?php
namespace KnitPay\Gateways\PayU;

use Pronamic\WordPress\Pay\AbstractGatewayIntegration;

/**
 * Title: PayU Integration
 * Copyright: 2020-2021 Knit Pay
 *
 * @author Knit Pay
 * @version 5.4.0
 * @since 5.4.0
 */
class Integration extends AbstractGatewayIntegration {


	/**
	 * Construct PayU integration.
	 *
	 * @param array $args
	 *            Arguments.
	 */
	public function __construct( $args = array() ) {
		$args = wp_parse_args(
			$args,
			array(
				'id'            => 'pay-u',
				'name'          => 'PayU',
				'url'           => 'https://www.knitpay.org/integrate-payu-payment-gateway-with-various-wordpress-plugins/?utm_source=knit-pay&utm_medium=ecommerce-module&utm_campaign=module-admin&utm_content=',
				'product_url'   => 'http://go.thearrangers.xyz/payu?utm_source=knit-pay&utm_medium=ecommerce-module&utm_campaign=module-admin&utm_content=product-url',
				'dashboard_url' => 'http://go.thearrangers.xyz/payu?utm_source=knit-pay&utm_medium=ecommerce-module&utm_campaign=module-admin&utm_content=dashboard-url',
				'provider'      => 'pay-u',
				'supports'      => array(
					'webhook',
					'webhook_log',
					'webhook_no_config',
				),
			// 'manual_url' => \__( 'http://go.thearrangers.xyz/payu', 'pronamic_ideal' ),
			)
		);

		parent::__construct( $args );
	}

	/**
	 * Get settings fields.
	 *
	 * @return array
	 */
	public function get_settings_fields() {
		$fields = array();

		if ( ! defined( 'KNIT_PAY_PAYU' ) ) {
			$fields[] = array(
				'section' => 'general',
				'type'    => 'html',
				'html'    => sprintf(
					/* translators: 1: PayU */
					__( 'Knit Pay supports %1$s with a Premium Addon. But you can get this premium addon for free.%2$s', 'knit-pay' ),
					__( 'PayU', 'knit-pay' ),
					'<br><br><a class="button button-primary" target="_blank" href="' . $this->get_url() . 'know-more"
                     role="button"><strong>Click Here to Know More</strong></a>'
				),
			);
			return $fields;
		}

		// Intro.
		$fields[] = array(
			'section' => 'general',
			'type'    => 'html',
			'html'    => sprintf(
				/* translators: 1: PayU */
				__( 'Account details are provided by %1$s after registration. These settings need to match with the %1$s dashboard.', 'pronamic_ideal' ),
				__( 'PayU', 'pronamic_ideal' )
			),
		);

		// Merchant Key
		$fields[] = array(
			'section'  => 'general',
			'filter'   => FILTER_SANITIZE_STRING,
			'meta_key' => '_pronamic_gateway_payu_merchant_key',
			'title'    => __( 'Merchant Key', 'knit-pay' ),
			'type'     => 'text',
			'classes'  => array(
				'regular-text',
				'code',
			),
			'tooltip'  => __( 'Merchant Key as mentioned in the PayU dashboard at the "Integration" page.', 'knit-pay' ),
		);

		// Merchant Salt
		$fields[] = array(
			'section'  => 'general',
			'filter'   => FILTER_SANITIZE_STRING,
			'meta_key' => '_pronamic_gateway_payu_merchant_salt',
			'title'    => __( 'Merchant Salt', 'knit-pay' ),
			'type'     => 'text',
			'classes'  => array(
				'regular-text',
				'code',
			),
			'tooltip'  => __( 'Merchant Salt as mentioned in the PayU dashboard at the "Integration" page.', 'knit-pay' ),
		);

		// Return fields.
		return $fields;
	}

	public function get_config( $post_id ) {
		$config = new Config();

		$config->merchant_key  = $this->get_meta( $post_id, 'payu_merchant_key' );
		$config->merchant_salt = $this->get_meta( $post_id, 'payu_merchant_salt' );
		$config->mode          = $this->get_meta( $post_id, 'mode' );

		return $config;
	}

	/**
	 * Get gateway.
	 *
	 * @param int $post_id
	 *            Post ID.
	 * @return Gateway
	 */
	public function get_gateway( $post_id ) {
		return new Gateway( $this->get_config( $post_id ) );
	}
}
